/*
 * Ind_WrapMulitObj.hpp
 *
 *  Created on: Mar 18, 2015
 *      Author: Joost Huizinga
 */

#ifndef IND_WRAPMULTIOBJ_HPP_
#define IND_WRAPMULTIOBJ_HPP_

#include <vector>

/**
 * Wrapper for individuals in a multi-objective algorithm.
 */
template<typename Indiv>
class IndivMultiObj: public Indiv
{
public:

    /**
     * Creates an instance of the multi-objective individual based on the parent type.
     *
     * @param other An instance of the parent type.
     */
    IndivMultiObj(const Indiv& other): Indiv(other){
        //nix
    }

    /**
     * Creates an instance of the multi-objective individual, implicitly calling the constructor
     * of the parent type.
     */
    IndivMultiObj(){
        //nix
    }

    /**
     * Adds an objective to this individual.
     *
     * @param obj The score on this particular objective. Note that, in most cases, objectives are maximized.
     */
    void addObjective(double obj){
        _objectives.push_back(obj);
    }

    /**
     * Returns a reference to the vector of objectives.
     */
    const std::vector<double>& getObjectives(){
        return _objectives;
    }

    /**
     * Returns the number of objectives stored with this individual.
     *
     * The same as: getObjectives().size()
     */
    size_t getNbOfObjectives() const{
        return _objectives.size();
    }

    /**
     * Returns a copy objective at the supplied index.
     *
     * Similar to: getObjectives()[index]
     *
     * @param index The index of the objective to be returned.
     */
    double getObjective(size_t index) const{
        return _objectives[index];
    }

    /**
     * Clears all objectives currently stored.
     */
    void clearObjectives(){
        _objectives.clear();
    }

private:
    std::vector<double> _objectives;
};

template<typename Indiv>
struct lexicalSort{
    bool operator()(const Indiv& indiv1, const Indiv& indiv2){
        for(size_t i=0; i<indiv1.getNbOfObjectives(); ++i){
            if(indiv1.getObjective(i) < indiv2.getObjective(i)){
                return true;
            } else if(indiv1.getObjective(i) > indiv2.getObjective(i)){
                return false;
            }
        }
        return false;
    }
};


#endif /* IND_WRAPMULTIOBJ_HPP_ */
